<?php

namespace Hrm\Vacancy\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Hrm\Vacancy\Http\Requests\VacancyRequest;
use Hrm\Vacancy\Interfaces\VacancyRepositoryInterface;
use Hrm\Vacancy\Models\Vacancy;

/**
 * Resource controller class for vacancy.
 */
class VacancyResourceController extends BaseController
{

    /**
     * Initialize vacancy resource controller.
     *
     * @param type VacancyRepositoryInterface $vacancy
     *
     * @return null
     */
    public function __construct(VacancyRepositoryInterface $vacancy)
    {
        parent::__construct();
        $this->repository = $vacancy;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Hrm\Vacancy\Repositories\Criteria\VacancyResourceCriteria::class);
    }

    /**
     * Display a list of vacancy.
     *
     * @return Response
     */
    public function index(VacancyRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Hrm\Vacancy\Repositories\Presenter\VacancyPresenter::class)
                ->$function();
        }

        $vacancies = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('vacancy::vacancy.names'))
            ->view('vacancy::vacancy.index', true)
            ->data(compact('vacancies', 'view'))
            ->output();
    }

    /**
     * Display vacancy.
     *
     * @param Request $request
     * @param Model   $vacancy
     *
     * @return Response
     */
    public function show(VacancyRequest $request, Vacancy $vacancy)
    {

        if ($vacancy->exists) {
            $view = 'vacancy::vacancy.show';
        } else {
            $view = 'vacancy::vacancy.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('vacancy::vacancy.name'))
            ->data(compact('vacancy'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new vacancy.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(VacancyRequest $request)
    {

        $vacancy = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('vacancy::vacancy.name')) 
            ->view('vacancy::vacancy.create', true) 
            ->data(compact('vacancy'))
            ->output();
    }

    /**
     * Create new vacancy.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(VacancyRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $vacancy                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('vacancy::vacancy.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('vacancy/vacancy/' . $vacancy->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/vacancy/vacancy'))
                ->redirect();
        }

    }

    /**
     * Show vacancy for editing.
     *
     * @param Request $request
     * @param Model   $vacancy
     *
     * @return Response
     */
    public function edit(VacancyRequest $request, Vacancy $vacancy)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('vacancy::vacancy.name'))
            ->view('vacancy::vacancy.edit', true)
            ->data(compact('vacancy'))
            ->output();
    }

    /**
     * Update the vacancy.
     *
     * @param Request $request
     * @param Model   $vacancy
     *
     * @return Response
     */
    public function update(VacancyRequest $request, Vacancy $vacancy)
    {
        try {
            $attributes = $request->all();

            $vacancy->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('vacancy::vacancy.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('vacancy/vacancy/' . $vacancy->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('vacancy/vacancy/' . $vacancy->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the vacancy.
     *
     * @param Model   $vacancy
     *
     * @return Response
     */
    public function destroy(VacancyRequest $request, Vacancy $vacancy)
    {
        try {

            $vacancy->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('vacancy::vacancy.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('vacancy/vacancy/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('vacancy/vacancy/' . $vacancy->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple vacancy.
     *
     * @param Model   $vacancy
     *
     * @return Response
     */
    public function delete(VacancyRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('vacancy::vacancy.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('vacancy/vacancy'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/vacancy/vacancy'))
                ->redirect();
        }

    }

    /**
     * Restore deleted vacancies.
     *
     * @param Model   $vacancy
     *
     * @return Response
     */
    public function restore(VacancyRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('vacancy::vacancy.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/vacancy/vacancy'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/vacancy/vacancy/'))
                ->redirect();
        }

    }

}
